import { BIO_ENTITY, DRUGS_CHEMICALS_MIRNA } from '@/constants';
import { STEP } from '@/constants/searchDrawer';
import {
  currentStepDrawerStateSelector,
  stepTypeDrawerSelector,
} from '@/redux/drawer/drawer.selectors';
import { useSelector } from 'react-redux';
import { ResultsList } from '@/components/Map/Drawer/SearchDrawerWrapper/ResultsList';
import { GroupedSearchResults } from '@/components/Map/Drawer/SearchDrawerWrapper/GroupedSearchResults';

export const SearchDrawerWrapper = (): JSX.Element => {
  const currentStep = useSelector(currentStepDrawerStateSelector);
  const stepType = useSelector(stepTypeDrawerSelector);

  const isBioEntityType = stepType === BIO_ENTITY;
  const isChemicalsDrugsOrMirnaType = DRUGS_CHEMICALS_MIRNA.includes(stepType);

  return (
    <div data-testid="search-drawer-content">
      {/* first step for displaying search results, drawers etc */}
      {currentStep === STEP.FIRST && <GroupedSearchResults />}
      {/* 2nd step for bioEntities aka content */}
      {currentStep === STEP.SECOND && isBioEntityType && (
        <div data-testid="search-second-step">The second step</div>
      )}
      {/* 2nd step for drugs,chemicals,mirna */}
      {currentStep === STEP.SECOND && isChemicalsDrugsOrMirnaType && (
        <div data-testid="search-second-step">
          <ResultsList />
        </div>
      )}
      {/* last step for bioentity */}
      {currentStep === STEP.THIRD && isBioEntityType && (
        <div data-testid="search-third-step">The third step</div>
      )}
      {/* last step for drugs,chemicals,mirna */}
      {currentStep === STEP.THIRD && isChemicalsDrugsOrMirnaType && (
        <div data-testid="search-third-step">The third step</div>
      )}
    </div>
  );
};
