import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen } from '@testing-library/react';
import { StoreType } from '@/redux/store';
import { MODELS_MOCK_SHORT } from '@/models/mocks/modelsMock';
import { openedDrawerSubmapsFixture } from '@/redux/drawer/drawerFixture';
import { SubmapsDrawer } from './SubmapsDrawer';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <SubmapsDrawer />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('SubmapsDrawer - component', () => {
  it('should display drawer heading and list of submaps', () => {
    renderComponent({
      models: { data: MODELS_MOCK_SHORT, loading: 'succeeded', error: { name: '', message: '' } },
    });

    expect(screen.getByText('Core PD map')).toBeInTheDocument();
    expect(screen.getByText('Histamine signaling')).toBeInTheDocument();
    expect(screen.getByText('PRKN substrates')).toBeInTheDocument();
  });
  it('should close drawer after clicking close button', () => {
    const { store } = renderComponent({
      models: { data: MODELS_MOCK_SHORT, loading: 'succeeded', error: { name: '', message: '' } },
      drawer: openedDrawerSubmapsFixture,
    });
    const closeButton = screen.getByRole('close-drawer-button');

    closeButton.click();

    const {
      drawer: { isOpen },
    } = store.getState();

    expect(isOpen).toBe(false);
  });
});
