/* eslint-disable no-magic-numbers */
import { MAP_DATA_INITIAL_STATE } from '@/redux/map/map.constants';
import mapSlice from '@/redux/map/map.slice';
import { getReduxWrapperUsingSliceReducer } from '@/utils/testing/getReduxWrapperUsingSliceReducer';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { renderHook, waitFor } from '@testing-library/react';
import { Map } from 'ol';
import TileLayer from 'ol/layer/Tile';
import React from 'react';
import { useOlMap } from '../useOlMap';
import { useOlMapLayers } from './useOlMapLayers';

const useRefValue = {
  current: null,
};

Object.defineProperty(useRefValue, 'current', {
  get: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
  set: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
});

jest.spyOn(React, 'useRef').mockReturnValue(useRefValue);

describe('useOlMapLayers - util', () => {
  it('should modify layers of the map instance on init', async () => {
    const { Wrapper } = getReduxWrapperUsingSliceReducer('map', mapSlice);
    const dummyElement = document.createElement('div');
    const mapInstance = new Map({ target: dummyElement });
    const setLayersSpy = jest.spyOn(mapInstance, 'setLayers');
    const CALLED_ONCE = 1;

    renderHook(() => useOlMapLayers({ mapInstance }), {
      wrapper: Wrapper,
    });

    await waitFor(() => expect(setLayersSpy).toBeCalledTimes(CALLED_ONCE));
  });

  it('should return valid View instance', async () => {
    const { Wrapper } = getReduxWrapperWithStore({
      map: {
        data: {
          ...MAP_DATA_INITIAL_STATE,
          size: {
            width: 256,
            height: 256,
            tileSize: 256,
            minZoom: 1,
            maxZoom: 1,
          },
          position: {
            initial: {
              x: 256,
              y: 256,
            },
            last: {
              x: 256,
              y: 256,
            },
          },
        },
        loading: 'idle',
        error: {
          name: '',
          message: '',
        },
      },
    });
    const dummyElement = document.createElement('div');
    const { result: hohResult } = renderHook(() => useOlMap({ target: dummyElement }), {
      wrapper: Wrapper,
    });

    const { result } = renderHook(
      () => useOlMapLayers({ mapInstance: hohResult.current.mapInstance }),
      {
        wrapper: Wrapper,
      },
    );

    expect(result.current[0]).toBeInstanceOf(TileLayer);
    expect(result.current[0].getSourceState()).toBe('ready');
  });
});
