/* eslint-disable no-magic-numbers */
import mapSlice, { setMapPosition } from '@/redux/map/map.slice';
import { getReduxWrapperUsingSliceReducer } from '@/utils/testing/getReduxWrapperUsingSliceReducer';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { renderHook, waitFor } from '@testing-library/react';
import { View } from 'ol';
import Map from 'ol/Map';
import React from 'react';
import { MAP_DATA_INITIAL_STATE } from '../../../../../redux/map/map.constants';
import { useOlMap } from '../useOlMap';
import { useOlMapView } from './useOlMapView';

const useRefValue = {
  current: null,
};

Object.defineProperty(useRefValue, 'current', {
  get: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
  set: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
});

jest.spyOn(React, 'useRef').mockReturnValue(useRefValue);

describe('useOlMapView - util', () => {
  it('should modify view of the map instance on INITIAL position config change', async () => {
    const { Wrapper, store } = getReduxWrapperUsingSliceReducer('map', mapSlice);
    const dummyElement = document.createElement('div');
    const { result: hohResult } = renderHook(() => useOlMap({ target: dummyElement }), {
      wrapper: Wrapper,
    });
    const setViewSpy = jest.spyOn(hohResult.current.mapInstance as Map, 'setView');
    const CALLED_ONCE = 1;

    store.dispatch(
      setMapPosition({
        position: {
          initial: {
            x: 0,
            y: 0,
          },
        },
      }),
    );

    renderHook(() => useOlMapView({ mapInstance: hohResult.current.mapInstance }), {
      wrapper: Wrapper,
    });

    await waitFor(() => expect(setViewSpy).toBeCalledTimes(CALLED_ONCE));
  });

  it('should return valid View instance', async () => {
    const { Wrapper } = getReduxWrapperWithStore({
      map: {
        data: {
          ...MAP_DATA_INITIAL_STATE,
          size: {
            width: 256,
            height: 256,
            tileSize: 256,
            minZoom: 1,
            maxZoom: 1,
          },
          position: {
            initial: {
              x: 256,
              y: 256,
            },
            last: {
              x: 256,
              y: 256,
            },
          },
        },
        loading: 'idle',
        error: {
          name: '',
          message: '',
        },
      },
    });
    const dummyElement = document.createElement('div');
    const { result: hohResult } = renderHook(() => useOlMap({ target: dummyElement }), {
      wrapper: Wrapper,
    });

    const { result } = renderHook(
      () => useOlMapView({ mapInstance: hohResult.current.mapInstance }),
      {
        wrapper: Wrapper,
      },
    );

    expect(result.current).toBeInstanceOf(View);
    expect(result.current.getCenter()).toStrictEqual([0, -0]);
  });
});
