import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { HttpStatusCode } from 'axios';
import { apiPath } from '@/redux/apiPath';
import { getBioEntity } from './bioEntity.thunks';
import contentsReducer from './bioEntity.slice';
import { BioEntityContentsState } from './bioEntity.types';

const mockedAxiosClient = mockNetworkNewAPIResponse();
const SEARCH_QUERY = 'park7';

describe('bioEntityContents thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioEntityContentsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('bioEntityContents', contentsReducer);
  });
  describe('getBioEntityContents', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient
        .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
        .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

      const { payload } = await store.dispatch(getBioEntity(SEARCH_QUERY));
      expect(payload).toEqual(bioEntityResponseFixture.content);
    });
    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getBioEntity(SEARCH_QUERY));
      expect(payload).toEqual(undefined);
    });
  });
});
