import { createAsyncThunk } from '@reduxjs/toolkit';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { BioEntityContent, BioEntityResponse } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { apiPath } from '@/redux/apiPath';
import { bioEntityResponseSchema } from '@/models/bioEntityResponseSchema';

export const getBioEntity = createAsyncThunk(
  'project/getBioEntityContents',
  async (searchQuery: string): Promise<BioEntityContent[] | undefined> => {
    const response = await axiosInstanceNewAPI.get<BioEntityResponse>(
      apiPath.getBioEntityContentsStringWithQuery(searchQuery),
    );

    const isDataValid = validateDataUsingZodSchema(response.data, bioEntityResponseSchema);

    return isDataValid ? response.data.content : undefined;
  },
);
