import { chemicalsFixture } from '@/models/fixtures/chemicalsFixture';
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import chemicalsReducer from './chemicals.slice';
import { getChemicals } from './chemicals.thunks';
import { ChemicalsState } from './chemicals.types';

const mockedAxiosClient = mockNetworkResponse();
const SEARCH_QUERY = 'Corticosterone';

const INITIAL_STATE: ChemicalsState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('chemicals reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<ChemicalsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('chemicals', chemicalsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(chemicalsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store after succesfull getChemicals query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getChemicalsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, chemicalsFixture);

    const { type } = await store.dispatch(getChemicals(SEARCH_QUERY));
    const { data, loading, error } = store.getState().chemicals;

    expect(type).toBe('project/getChemicals/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(chemicalsFixture);
  });

  it('should update store after failed getChemicals query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getChemicalsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.NotFound, chemicalsFixture);

    const { type } = await store.dispatch(getChemicals(SEARCH_QUERY));
    const { data, loading, error } = store.getState().chemicals;

    expect(type).toBe('project/getChemicals/rejected');
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
  });

  it('should update store on loading getChemicals query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getChemicalsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, chemicalsFixture);

    const chemicalsPromise = store.dispatch(getChemicals(SEARCH_QUERY));

    const { data, loading } = store.getState().chemicals;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    chemicalsPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().chemicals;

      expect(dataPromiseFulfilled).toEqual(chemicalsFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
