import { ActionReducerMapBuilder } from '@reduxjs/toolkit';
import { getPointMerged } from '../../utils/object/getPointMerged';
import { initMapData, initMapPosition } from './map.thunks';
import { MapState, SetMapDataAction, SetMapPositionDataAction } from './map.types';

export const setMapDataReducer = (state: MapState, action: SetMapDataAction): void => {
  const payload = action.payload || {};

  state.data = {
    ...state.data,
    ...payload,
  };
};

export const setMapPositionReducer = (state: MapState, action: SetMapPositionDataAction): void => {
  const payload = action.payload || {};
  const payloadPosition = 'position' in payload ? payload.position : undefined;
  const statePosition = state.data.position;

  state.data = {
    ...state.data,
    position: {
      initial: getPointMerged(payloadPosition?.initial || {}, statePosition.initial),
      last: getPointMerged(payloadPosition?.last || {}, statePosition.last),
    },
  };
};

export const getMapReducers = (builder: ActionReducerMapBuilder<MapState>): void => {
  builder.addCase(initMapData.pending, state => {
    state.loading = 'pending';
  });
  builder.addCase(initMapData.fulfilled, (state, action) => {
    const payload = action.payload || {};
    state.data = { ...state.data, ...payload };
    state.loading = 'succeeded';
  });
  builder.addCase(initMapData.rejected, state => {
    state.loading = 'failed';
    // TODO to discuss manage state of failure
  });
};

export const getMapPositionReducers = (builder: ActionReducerMapBuilder<MapState>): void => {
  builder.addCase(initMapPosition.pending, state => {
    state.loading = 'pending';
  });
  builder.addCase(initMapPosition.fulfilled, (state, action) => {
    const payload = action.payload || {};
    state.data = { ...state.data, ...payload };
    state.loading = 'succeeded';
  });
  builder.addCase(initMapPosition.rejected, state => {
    state.loading = 'failed';
    // TODO to discuss manage state of failure
  });
};
