import { PROJECT_ID } from '@/constants';
import { QueryData } from '@/types/query';
import { GetUpdatedMapDataResult, getUpdatedMapData } from '@/utils/map/getUpdatedMapData';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { backgroundsDataSelector } from '../backgrounds/background.selectors';
import { getAllBackgroundsByProjectId } from '../backgrounds/backgrounds.thunks';
import { modelsDataSelector } from '../models/models.selectors';
import { getModels } from '../models/models.thunks';
import { getAllPublicOverlaysByProjectId } from '../overlays/overlays.thunks';
import type { AppDispatch, RootState } from '../store';
import {
  InitMapDataActionParams,
  InitMapDataActionPayload,
  SetMapPositionDataActionPayload,
} from './map.types';

const getInitMapDataPayload = (
  state: RootState,
  queryData: QueryData,
): GetUpdatedMapDataResult | object => {
  const FIRST = 0;
  const models = modelsDataSelector(state);
  const backgrounds = backgroundsDataSelector(state);
  const modelId = queryData?.modelId || models?.[FIRST]?.idObject;
  const backgroundId = queryData?.backgroundId || backgrounds?.[FIRST]?.id;
  const model = models.find(({ idObject }) => idObject === modelId);
  const background = backgrounds.find(({ id }) => id === backgroundId);
  const position = queryData?.initialPosition;

  if (!model || !background) {
    return {};
  }

  return getUpdatedMapData({
    model,
    background,
    position: {
      last: position,
      initial: position,
    },
  });
};

export const initMapData = createAsyncThunk<
  InitMapDataActionPayload,
  InitMapDataActionParams,
  { dispatch: AppDispatch; state: RootState }
>(
  'map/initMapData',
  async ({ queryData }, { dispatch, getState }): Promise<InitMapDataActionPayload> => {
    await Promise.all([
      dispatch(getAllBackgroundsByProjectId(PROJECT_ID)),
      dispatch(getAllPublicOverlaysByProjectId(PROJECT_ID)),
      dispatch(getModels()),
    ]);

    const state = getState();
    return getInitMapDataPayload(state, queryData);
  },
);

export const initMapPosition = createAsyncThunk<
  InitMapDataActionPayload,
  InitMapDataActionParams,
  { dispatch: AppDispatch; state: RootState }
>(
  'map/initMapPosition',
  async ({ queryData }, { getState }): Promise<SetMapPositionDataActionPayload> => {
    const state = getState();
    return getInitMapDataPayload(state, queryData);
  },
);
