import { FetchDataState } from '@/types/fetchDataState';
import { Point } from '@/types/map';
import { QueryData } from '@/types/query';
import { PayloadAction } from '@reduxjs/toolkit';

export interface MapSize {
  width: number;
  height: number;
  tileSize: number;
  minZoom: number;
  maxZoom: number;
}

export type MapData = {
  projectId: string;
  meshId: string;
  modelId: number;
  backgroundId: number;
  overlaysIds: number[];
  size: MapSize;
  position: {
    initial: Point;
    last: Point;
  };
  show: {
    legend: boolean;
    comments: boolean;
  };
};

export type MapState = FetchDataState<MapData, MapData>;

export type SetMapDataActionPayload =
  | (Omit<Partial<MapData>, 'position' | 'projectId'> & {
      projectId?: string;
    })
  | undefined;

export type SetMapDataAction = PayloadAction<SetMapDataActionPayload>;

export type InitMapDataActionParams = { queryData: QueryData };

export type InitMapDataActionPayload = SetMapDataActionPayload | object;

export type InitMapDataAction = PayloadAction<SetMapDataAction>;

export type MiddlewareAllowedAction = PayloadAction<
  SetMapDataActionPayload | InitMapDataActionPayload
>;

export type SetMapDataByQueryDataActionParams = { queryData: QueryData };

export type SetMapDataByQueryDataActionPayload = Pick<
  MapData,
  'modelId' | 'backgroundId' | 'position'
>;

export type SetMapPositionDataActionPayload = Pick<MapData, 'position'> | object;

export type SetMapPositionDataAction = PayloadAction<SetMapPositionDataActionPayload>;
