import { currentBackgroundSelector } from '@/redux/backgrounds/background.selectors';
import type { AppDispatch, AppListenerEffectAPI, AppStartListening } from '@/redux/store';
import { GetUpdatedMapDataResult, getUpdatedMapData } from '@/utils/map/getUpdatedMapData';
import { Action, createListenerMiddleware } from '@reduxjs/toolkit';
import { setMapData, setMapPosition } from '../map.slice';
import { checkIfIsMapUpdateActionValid } from './checkIfIsMapUpdateActionValid';
import { getUpdatedModel } from './getUpdatedModel';

export const mapListenerMiddleware = createListenerMiddleware();

const startListening = mapListenerMiddleware.startListening as AppStartListening;

/* prettier-ignore */
export const dispatchMapDataWithPosition =
  (updatedMapData: GetUpdatedMapDataResult) =>
    (dispatch: AppDispatch): void => {
      dispatch(setMapData(updatedMapData));
      dispatch(setMapPosition(updatedMapData));
    };

export const mapDataMiddlewareListener = async (
  action: Action,
  { getOriginalState, dispatch }: AppListenerEffectAPI,
): Promise<void> => {
  const state = getOriginalState();
  const updatedModel = getUpdatedModel(action, state);
  const isActionValid = checkIfIsMapUpdateActionValid(action, state);

  if (!updatedModel || !isActionValid) {
    return;
  }

  const background = currentBackgroundSelector(state);
  const updatedMapData = getUpdatedMapData({ model: updatedModel, background });
  dispatch(setMapData(updatedMapData));
  dispatch(setMapPosition(updatedMapData));
};

startListening({
  type: 'map/setMapData',
  effect: mapDataMiddlewareListener,
});
