import { mirnasFixture } from '@/models/fixtures/mirnasFixture';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { HttpStatusCode } from 'axios';
import { apiPath } from '@/redux/apiPath';
import { getMirnas } from './mirnas.thunks';
import mirnasReducer from './mirnas.slice';
import { MirnasState } from './mirnas.types';

const mockedAxiosClient = mockNetworkResponse();
const SEARCH_QUERY = 'hsa-miR-302b-3p';

const INITIAL_STATE: MirnasState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('mirnas reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<MirnasState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('mirnas', mirnasReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(mirnasReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store after succesfull getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, mirnasFixture);

    const { type } = await store.dispatch(getMirnas(SEARCH_QUERY));
    const { data, loading, error } = store.getState().mirnas;

    expect(type).toBe('project/getMirnas/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(mirnasFixture);
  });

  it('should update store after failed getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.NotFound, []);

    const { type } = await store.dispatch(getMirnas(SEARCH_QUERY));
    const { data, loading, error } = store.getState().mirnas;

    expect(type).toBe('project/getMirnas/rejected');
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
  });

  it('should update store on loading getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, mirnasFixture);

    const mirnasPromise = store.dispatch(getMirnas(SEARCH_QUERY));

    const { data, loading } = store.getState().mirnas;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    mirnasPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().mirnas;

      expect(dataPromiseFulfilled).toEqual(mirnasFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
