import { mapModelSchema } from '@/models/modelSchema';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { MapModel } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';

export const getModels = createAsyncThunk(
  'project/getModels',
  async (): Promise<MapModel[] | undefined> => {
    const response = await axiosInstance.get<MapModel[]>(apiPath.getModelsString());

    const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapModelSchema));

    return isDataValid ? response.data : undefined;
  },
);
