import { PROJECT_ID } from '@/constants';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { initMapData, initMapPosition } from '@/redux/map/map.thunks';
import { getProjectById } from '@/redux/project/project.thunks';
import { initDataLoadingInitialized } from '@/redux/root/init.selectors';
import { AppDispatch } from '@/redux/store';
import { QueryData } from '@/types/query';
import { useRouter } from 'next/router';
import { useEffect } from 'react';
import { useSelector } from 'react-redux';
import { getQueryData } from '../query-manager/getQueryData';

interface GetInitStoreDataArgs {
  queryData: QueryData;
}

/* prettier-ignore */
export const getInitStoreData =
  ({ queryData }: GetInitStoreDataArgs) =>
    (dispatch: AppDispatch): void => {
      dispatch(getProjectById(PROJECT_ID));
      dispatch(initMapData({ queryData }));
      dispatch(initMapPosition({ queryData }));
    };

export const useInitializeStore = (): void => {
  const dispatch = useAppDispatch();
  const isInitialized = useSelector(initDataLoadingInitialized);
  const { query, isReady: isRouterReady } = useRouter();

  useEffect(() => {
    const isQueryReady = query && isRouterReady;
    if (isInitialized || !isQueryReady) {
      return;
    }

    dispatch(
      getInitStoreData({
        queryData: getQueryData(query),
      }),
    );
  }, [dispatch, query, isInitialized, isRouterReady]);
};
