/* eslint-disable no-magic-numbers */
import { QueryData } from '@/types/query';
import { ParsedUrlQuery } from 'querystring';
import { getQueryData, getQueryFieldNumberCurry } from './getQueryData';

describe('getQueryFieldNumber - util', () => {
  const query: ParsedUrlQuery = {
    numberString: '123',
    stringString: 'abcd',
    emptyString: '',
  };

  const getQueryFieldNumber = getQueryFieldNumberCurry(query);

  it('should return the number on key value is number string', () => {
    expect(getQueryFieldNumber('numberString')).toBe(123);
  });

  it('should return undefined on key value is non-number string', () => {
    expect(getQueryFieldNumber('stringString')).toBe(undefined);
  });

  it('should return undefined on key value is empty', () => {
    expect(getQueryFieldNumber('emptyString')).toBe(undefined);
  });
});

describe('getQueryData - util', () => {
  it('should return valid query data on valid query params', () => {
    const queryParams: ParsedUrlQuery = {
      x: '2354',
      y: '5321',
      z: '6',
      modelId: '54',
      backgroundId: '13',
    };

    const queryData: QueryData = {
      modelId: 54,
      backgroundId: 13,
      initialPosition: {
        x: 2354,
        y: 5321,
        z: 6,
      },
    };

    expect(getQueryData(queryParams)).toMatchObject(queryData);
  });

  it('should return partial query data on partial query params', () => {
    const queryParams: ParsedUrlQuery = {
      x: '2354',
      modelId: '54',
    };

    const queryData: QueryData = {
      modelId: 54,
      backgroundId: undefined,
      initialPosition: {
        x: 2354,
        y: undefined,
        z: undefined,
      },
    };

    expect(getQueryData(queryParams)).toMatchObject(queryData);
  });
});
